#include "TROOT.h"
#include "TFile.h"
#include "TTree.h"
#include "TH1.h"
#include "TH2.h"
#include "TLorentzVector.h"
#include "TMath.h"

void plot_angular_distributions(string file_name, string tree_name, string particle_decay){

  TFile* f = new TFile(file_name.c_str());
  TTree* m_inTree = (TTree*) f->Get(tree_name.c_str());

  Double_t run_number;
  Double_t polarization=0.;

  Double_t beam_energy;

  Double_t p_px;
  Double_t p_py;
  Double_t p_pz;
  Double_t p_E;

  Double_t kp_px;
  Double_t kp_py;
  Double_t kp_pz;
  Double_t kp_E;

  Double_t km_px;
  Double_t km_py;
  Double_t km_pz;
  Double_t km_E;

  Double_t pi0_px;
  Double_t pi0_py;
  Double_t pi0_pz;
  Double_t pi0_E;

  Double_t m_kpkmpi0;
  Double_t m_kppi0;
  Double_t m_kmpi0;
  Double_t m_kpkm;
  Double_t t_mandelstam;

  Double_t weight;

  m_inTree->SetBranchAddress( "run_number", &run_number);
  m_inTree->SetBranchAddress( "polarization", &polarization);

  m_inTree->SetBranchAddress( "beam_energy", &beam_energy);

  m_inTree->SetBranchAddress( "p_px", &p_px );
  m_inTree->SetBranchAddress( "p_py", &p_py );
  m_inTree->SetBranchAddress( "p_pz", &p_pz );
  m_inTree->SetBranchAddress( "p_E", &p_E );

  m_inTree->SetBranchAddress( "kp_px", &kp_px );
  m_inTree->SetBranchAddress( "kp_py", &kp_py );
  m_inTree->SetBranchAddress( "kp_pz", &kp_pz );
  m_inTree->SetBranchAddress( "kp_E", &kp_E );

  m_inTree->SetBranchAddress( "km_px", &km_px );
  m_inTree->SetBranchAddress( "km_py", &km_py );
  m_inTree->SetBranchAddress( "km_pz", &km_pz );
  m_inTree->SetBranchAddress( "km_E", &km_E );

  m_inTree->SetBranchAddress( "pi0_px", &pi0_px );
  m_inTree->SetBranchAddress( "pi0_py", &pi0_py );
  m_inTree->SetBranchAddress( "pi0_pz", &pi0_pz );
  m_inTree->SetBranchAddress( "pi0_E", &pi0_E );

  m_inTree->SetBranchAddress( "m_kpkmpi0", &m_kpkmpi0);
  m_inTree->SetBranchAddress( "m_kppi0", &m_kppi0);
  m_inTree->SetBranchAddress( "m_kmpi0", &m_kmpi0);
  m_inTree->SetBranchAddress( "m_kpkm", &m_kpkm);
  m_inTree->SetBranchAddress( "t_mandelstam", &t_mandelstam);

  m_inTree->SetBranchAddress( "weight", &weight );  

  TH1D* hcosTheta = new TH1D("hcosTheta","",100,-1.,1.);
  TH1D* hphi = new TH1D("hphi","",360,-180.,180.);
  TH1D* hcosTheta_H = new TH1D("hcosTheta_H","",100,-1.,1.);
  TH1D* hphi_H = new TH1D("hphi_H","",360,-180.,180.);
  TH1D* hPhi = new TH1D("hPhi","",360,-180.,180.);
  TH1D* hPhi2 = new TH1D("hPhi2","",360,-180.,180.);

  TH2D* hkpkmpi0_cosTheta = new TH2D("hkpkmpi0_cosTheta","",100,1.,3.0,100,-1.,1.);
  TH2D* hkpkmpi0_phi = new TH2D("hkpkmpi0_phi","",100,1.,3.0,360,-180.,180.);
  TH2D* hkpkmpi0_cosTheta_H = new TH2D("hkpkmpi0_cosTheta_H","",100,1.,3.0,100,-1.,1.);
  TH2D* hkpkmpi0_phi_H = new TH2D("hkpkmpi0_phi_H","",100,1.,3.0,360,-180.,180.);
  TH2D* hkpkmpi0_Phi = new TH2D("hkpkmpi0_Phi","",100,1.,3.0,360,-180.,180.);

  Double_t polAngle = polarization;

  if (polarization == -1.) polarization = 0.;

  for (Long64_t i=0; i<m_inTree->GetEntries(); i++)
  {
    m_inTree->GetEntry(i);
    if (beam_energy < 8.2 || beam_energy > 8.8) continue;
    //if (t_mandelstam < 1.0) continue;
    TLorentzVector beam(0.,0.,beam_energy,beam_energy);
    TLorentzVector recoil(p_px,p_py,p_pz,p_E);
    TLorentzVector p1_temp(kp_px,kp_py,kp_pz,kp_E);
    TLorentzVector p2_temp(km_px,km_py,km_pz,km_E);
    TLorentzVector p3_temp(pi0_px,pi0_py,pi0_pz,pi0_E);

    TLorentzVector p1,p2,p3;

    if (particle_decay == "kppi0_km")
    {
      if (m_kppi0 < 0.84 || m_kppi0 > 0.94) continue;
      p1 = p1_temp;
      p2 = p3_temp;
      p3 = p2_temp;
    }
    else if (particle_decay == "kmpi0_kp")
    {
      if (m_kmpi0 < 0.84 || m_kmpi0 > 0.94) continue;
      p1 = p2_temp;
      p2 = p3_temp;
      p3 = p1_temp;
    }
    else if (particle_decay == "a0_pi0")
    {
      if (m_kpkm > 1.01) continue;
      p1 = p1_temp;
      p2 = p2_temp;
      p3 = p3_temp;
    }
    else if (particle_decay == "phi_pi0")
    {
      if (m_kpkm < 1.01 || m_kpkm > 1.03) continue;
      p1 = p1_temp;
      p2 = p2_temp;
      p3 = p3_temp;
    }
/**    else if (particle_decay == "kstar")
    {
      if ((m_kmpi0 < 0.84 || m_kmpi0 > 0.94) && (m_kppi0 < 0.84 || m_kppi0 > 0.94)) continue;
      if (m_kmpi0 > 0.84 && m_kmpi0 < 0.94)
      {
        p1 = p2_temp;
        p2 = p3_temp;
        p3 = p1_temp;
      }
      else if (m_kppi0 > 0.84 && m_kppi0 < 0.94)
      {
        p1 = p1_temp;
        p2 = p3_temp;
        p3 = p2_temp;
      }
    }**/
    else
    {
      return;
    }

    TLorentzVector resonance = p1 + p2 + p3;
    TLorentzVector daughter1 = p1 + p2;
    TLorentzVector daughter2 = p1;
    TLorentzVector daughter3 = p2;

    TLorentzVector target(0,0,0,0.938);
    TLorentzVector cm = beam+target;

    TVector3 cmRestBoost = cm.BoostVector();
    TLorentzVector beam_cm = beam;
    TLorentzVector recoil_cm = recoil;
    TLorentzVector resonance_cm = resonance;
    TLorentzVector daughter1_cm = daughter1;
    TLorentzVector daughter2_cm = daughter2;
    TLorentzVector daughter3_cm = daughter3;
    beam_cm.Boost(-1.0*cmRestBoost);
    recoil_cm.Boost(-1.0*cmRestBoost);
    resonance_cm.Boost(-1.0*cmRestBoost);
    daughter1_cm.Boost(-1.0*cmRestBoost);
    daughter2_cm.Boost(-1.0*cmRestBoost);
    daughter3_cm.Boost(-1.0*cmRestBoost);

    TVector3 resRestBoost = resonance_cm.BoostVector();
    TLorentzVector resonance_res = resonance_cm;
    TLorentzVector daughter1_res = daughter1_cm;
    TLorentzVector daughter2_res = daughter2_cm;
    TLorentzVector daughter3_res = daughter3_cm;
    resonance_res.Boost(-1.0*resRestBoost);
    daughter1_res.Boost(-1.0*resRestBoost);
    daughter2_res.Boost(-1.0*resRestBoost);
    daughter3_res.Boost(-1.0*resRestBoost);

    TVector3 resRestBoost2 = resonance.BoostVector();
    TLorentzVector p_recoil_res2 = recoil;
    TLorentzVector resonance_res2 = resonance;
    p_recoil_res2.Boost(-1.0*resRestBoost2);
    resonance_res2.Boost(-1.0*resRestBoost2);

    TVector3 vRestBoost = daughter1_res.BoostVector();
    TLorentzVector daughter2_v = daughter2_res;
    TLorentzVector daughter3_v = daughter3_res;
    daughter2_v.Boost(-1.0*vRestBoost);
    daughter3_v.Boost(-1.0*vRestBoost);

    TVector3 beam_cm_vec = beam_cm.Vect().Unit();
    TVector3 daughter1_res_vec = daughter1_res.Vect().Unit();

    TVector3 z = resonance_cm.Vect().Unit();
    TVector3 y = (beam_cm_vec.Cross(z)).Unit();
    TVector3 x = (y.Cross(z)).Unit();

    TVector3 Angles(daughter1_res_vec.Dot(x),daughter1_res_vec.Dot(y),daughter1_res_vec.Dot(z));

    double cosTheta = Angles.CosTheta();
    double phi = Angles.Phi();

    TVector3 eps(cos(polAngle*TMath::DegToRad()), sin(polAngle*TMath::DegToRad()), 0.0); // beam polarization vector
    double Phi = atan2(y.Dot(eps), beam.Vect().Unit().Dot(eps.Cross(y)));

    TVector3 daughter2_vec = daughter2_v.Vect().Unit();
    TVector3 daughter3_vec = daughter3_v.Vect().Unit();
    //TVector3 n_H = daughter2_vec.Cross(daughter3_vec);

    TVector3 z_H = daughter1_res.Vect().Unit();
    TVector3 y_H = z.Cross(z_H);
    TVector3 x_H = y_H.Cross(z_H);

    TVector3 Angles_H(daughter2_vec.Dot(x_H),daughter2_vec.Dot(y_H),daughter2_vec.Dot(z_H));

    double cosTheta_H = Angles_H.CosTheta();
    double phi_H = Angles_H.Phi();

    hkpkmpi0_cosTheta->Fill(m_kpkmpi0,cosTheta,weight);
    hkpkmpi0_phi->Fill(m_kpkmpi0,phi*TMath::RadToDeg(),weight);
    hkpkmpi0_cosTheta_H->Fill(m_kpkmpi0,cosTheta_H,weight);
    hkpkmpi0_phi_H->Fill(m_kpkmpi0,phi_H*TMath::RadToDeg(),weight);
    hkpkmpi0_Phi->Fill(m_kpkmpi0,Phi*TMath::RadToDeg(),weight);

    if (m_kpkmpi0 > 1.6) continue;

    hcosTheta->Fill(cosTheta,weight);
    hphi->Fill(phi*TMath::RadToDeg(),weight);
    hcosTheta_H->Fill(cosTheta_H,weight);
    hphi_H->Fill(phi_H*TMath::RadToDeg(),weight);
    hPhi->Fill(Phi*TMath::RadToDeg(),weight);
  }

  hcosTheta->SetDirectory(0);
  hphi->SetDirectory(0);
  hcosTheta_H->SetDirectory(0);
  hphi_H->SetDirectory(0);
  hPhi->SetDirectory(0);
  hPhi2->SetDirectory(0);

  hkpkmpi0_cosTheta->SetDirectory(0);
  hkpkmpi0_phi->SetDirectory(0);
  hkpkmpi0_cosTheta_H->SetDirectory(0);
  hkpkmpi0_phi_H->SetDirectory(0);
  hkpkmpi0_Phi->SetDirectory(0);

  f->Close();

  TFile* outFile = new TFile("outFile.root","RECREATE");
  outFile->cd();
  hcosTheta->Write();
  hphi->Write();
  hcosTheta_H->Write();
  hphi_H->Write();
  hPhi->Write();
  hPhi2->Write();
  hkpkmpi0_cosTheta->Write();
  hkpkmpi0_phi->Write();
  hkpkmpi0_cosTheta_H->Write();
  hkpkmpi0_phi_H->Write();
  hkpkmpi0_Phi->Write();
  outFile->Close();
}
